@@ -14,7 +14,7 @@ import (
 	"github.com/ubuntu/adsys"
 	"github.com/ubuntu/adsys/internal/adsysservice"
 	"github.com/ubuntu/adsys/internal/cmdhandler"
-	"github.com/ubuntu/adsys/internal/config"
+	"github.com/ubuntu/adsys/internal/consts"
 	log "github.com/ubuntu/adsys/internal/grpc/logstreamer"
 	"github.com/ubuntu/adsys/internal/i18n"
 )
@@ -29,13 +29,18 @@ func (a *App) installPolicy() {
 
 	var distro *string
 	mainCmd := &cobra.Command{
-		Use:       "admx lts-only|all",
-		Short:     i18n.G("Dump windows policy definitions"),
-		ValidArgs: []string{"lts-only", "all"},
-		Args:      cobra.ExactValidArgs(1),
-		RunE:      func(cmd *cobra.Command, args []string) error { return a.getPolicyDefinitions(args[0], *distro) },
+		Use:   "admx lts-only|all",
+		Short: i18n.G("Dump windows policy definitions"),
+		Args:  cobra.ExactValidArgs(1),
+		ValidArgsFunction: func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+			if len(args) != 0 {
+				return nil, cobra.ShellCompDirectiveNoFileComp
+			}
+			return []string{"lts-only", "all"}, cobra.ShellCompDirectiveNoFileComp
+		},
+		RunE: func(cmd *cobra.Command, args []string) error { return a.getPolicyDefinitions(args[0], *distro) },
 	}
-	distro = mainCmd.Flags().StringP("distro", "", config.DistroID, i18n.G("distro for which to retrieve policy definition."))
+	distro = mainCmd.Flags().StringP("distro", "", consts.DistroID, i18n.G("distro for which to retrieve policy definition."))
 	policyCmd.AddCommand(mainCmd)
 
 	var details, all, nocolor *bool
@@ -43,6 +48,13 @@ func (a *App) installPolicy() {
 		Use:   "applied [USER_NAME]",
 		Short: i18n.G("Print last applied GPOs for current or given user/machine"),
 		Args:  cmdhandler.ZeroOrNArgs(1),
+		ValidArgsFunction: func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+			if len(args) != 0 {
+				return nil, cobra.ShellCompDirectiveNoFileComp
+			}
+
+			return a.completeWithConnectedUsers()
+		},
 		RunE: func(cmd *cobra.Command, args []string) error {
 			var target string
 			if len(args) > 0 {
@@ -57,11 +69,45 @@ func (a *App) installPolicy() {
 	policyCmd.AddCommand(appliedCmd)
 	cmdhandler.RegisterAlias(appliedCmd, &a.rootCmd)
 
+	debugCmd := &cobra.Command{
+		Use:    "debug",
+		Short:  i18n.G("Debug various policy infos"),
+		Hidden: true,
+		Args:   cmdhandler.SubcommandsRequiredWithSuggestions,
+		RunE:   cmdhandler.NoCmd,
+	}
+	policyCmd.AddCommand(debugCmd)
+	gpoListCmd := &cobra.Command{
+		Use:               "gpolist-script",
+		Short:             i18n.G("Write GPO list python embeeded script in current directory"),
+		Args:              cobra.NoArgs,
+		ValidArgsFunction: cmdhandler.NoValidArgs,
+		RunE:              func(cmd *cobra.Command, args []string) error { return a.dumpGPOListScript() },
+	}
+	debugCmd.AddCommand(gpoListCmd)
+
 	var updateMachine, updateAll *bool
 	updateCmd := &cobra.Command{
 		Use:   "update [USER_NAME KERBEROS_TICKET_PATH]",
 		Short: i18n.G("Updates/Create a policy for current user or given user with its kerberos ticket"),
 		Args:  cmdhandler.ZeroOrNArgs(2),
+		ValidArgsFunction: func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+			// All and machine options don’t take arguments
+			if *updateAll || *updateMachine {
+				return nil, cobra.ShellCompDirectiveNoFileComp
+			}
+			switch len(args) {
+			case 0:
+				// Get all connected users
+				return a.completeWithConnectedUsers()
+			case 1:
+				// The user has already been process, let then specifying the ticket path
+				return nil, cobra.ShellCompDirectiveDefault
+			}
+
+			// We already have our 2 args: no more arg completion
+			return nil, cobra.ShellCompDirectiveNoFileComp
+		},
 		RunE: func(cmd *cobra.Command, args []string) error {
 			var user, krb5cc string
 			if len(args) > 0 {
@@ -168,6 +214,26 @@ func (a *App) dumpPolicies(target string, showDetails, showOverridden, nocolor b
 	return nil
 }
 
+func (a *App) dumpGPOListScript() error {
+	client, err := adsysservice.NewClient(a.config.Socket, a.getTimeout())
+	if err != nil {
+		return err
+	}
+	defer client.Close()
+
+	stream, err := client.GPOListScript(a.ctx, &adsys.Empty{})
+	if err != nil {
+		return err
+	}
+
+	script, err := singleMsg(stream)
+	if err != nil {
+		return err
+	}
+
+	return os.WriteFile("adsys-gpolist", []byte(script), 0750)
+}
+
 func colorizePolicies(policies string) (string, error) {
 	first := true
 	var out stringsBuilderWithError
@@ -262,8 +328,8 @@ func (a *App) update(isComputer, updateAll bool, target, krb5cc string) error {
 	}
 	defer client.Close()
 
-	// override for computer
-	if (isComputer || updateAll) && target == "" {
+	// get target for computer
+	if isComputer && target == "" {
 		hostname, err := os.Hostname()
 		if err != nil {
 			return err
@@ -276,7 +342,7 @@ func (a *App) update(isComputer, updateAll bool, target, krb5cc string) error {
 	}
 
 	// Update for current user
-	if target == "" {
+	if target == "" && !updateAll {
 		u, err := user.Current()
 		if err != nil {
 			return fmt.Errorf("failed to retrieve current user: %v", err)
@@ -300,3 +366,21 @@ func (a *App) update(isComputer, updateAll bool, target, krb5cc string) error {
 
 	return nil
 }
+
+func (a App) completeWithConnectedUsers() ([]string, cobra.ShellCompDirective) {
+	client, err := adsysservice.NewClient(a.config.Socket, a.getTimeout())
+	if err != nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+	defer client.Close()
+	stream, err := client.ListActiveUsers(a.ctx, &adsys.Empty{})
+	if err != nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+	list, err := singleMsg(stream)
+	if err != nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
+	return strings.Split(list, " "), cobra.ShellCompDirectiveNoFileComp
+}
