@@ -19,15 +19,22 @@ import (
 func TestNew(t *testing.T) {
 	t.Parallel()
 
+	bus := ad.GetSystemBus(t)
+
 	tests := map[string]struct {
-		cacheDirRO bool
-		runDirRO   bool
+		cacheDirRO      bool
+		runDirRO        bool
+		staticServerURL string
+		wantServerURL   string
 
 		wantErr bool
 	}{
-		"create one AD object will create all necessary cache dirs": {},
-		"failed to create KRB5 cache directory":                     {runDirRO: true, wantErr: true},
-		"failed to create GPO cache directory":                      {cacheDirRO: true, wantErr: true},
+		"create one AD object will create all necessary cache dirs": {staticServerURL: "ldap://my-static-server.example.com:1636/", wantServerURL: "ldap://my-static-server.example.com:1636/"},
+		"static server is always prefixed with ldap":                {staticServerURL: "my-static-server.example.com:1636/", wantServerURL: "ldap://my-static-server.example.com:1636/"},
+		"not provided static server URL is blank":                   {staticServerURL: "", wantServerURL: ""},
+
+		"failed to create KRB5 cache directory": {runDirRO: true, wantErr: true},
+		"failed to create GPO cache directory":  {cacheDirRO: true, wantErr: true},
 	}
 	for name, tc := range tests {
 		tc := tc
@@ -43,21 +50,24 @@ func TestNew(t *testing.T) {
 				require.NoError(t, os.Chmod(cacheDir, 0400), "Setup: can’t set cache directory to Read only")
 			}
 
-			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "localdomain",
+			adc, err := ad.New(context.Background(), tc.staticServerURL, "localdomain", bus,
 				ad.WithRunDir(runDir),
 				ad.WithCacheDir(cacheDir),
 				ad.WithSSSCacheDir("testdata/sss/db"))
 			if tc.wantErr {
 				require.NotNil(t, err, "AD creation should have failed")
-			} else {
-				require.NoError(t, err, "AD creation should be successful failed")
+				return
 			}
 
-			if !tc.wantErr {
-				// Ensure cache directories exists
-				assert.DirExists(t, adc.Krb5CacheDir(), "Kerberos ticket cache directory doesn't exist")
-				assert.DirExists(t, adc.GpoCacheDir(), "GPO cache directory doesn't exist")
-			}
+			require.NoError(t, err, "AD creation should be successful failed")
+
+			// Ensure cache directories exists
+			assert.DirExists(t, adc.Krb5CacheDir(), "Kerberos ticket cache directory doesn't exist")
+			assert.DirExists(t, adc.GpoCacheDir(), "GPO cache directory doesn't exist")
+
+			adServerURL, offline := adc.GetStatus()
+			assert.Equal(t, tc.wantServerURL, adServerURL, "AD server URl matches static configuration")
+			assert.False(t, offline, "Considered online until first update")
 		})
 	}
 }
@@ -68,6 +78,8 @@ func TestGetPolicies(t *testing.T) {
 	hostname, err := os.Hostname()
 	require.NoError(t, err, "Setup: failed to get hostname")
 
+	bus := ad.GetSystemBus(t)
+
 	standardGPO := entry.GPO{ID: "standard", Name: "standard-name", Rules: map[string][]entry.Entry{
 		"dconf": {
 			{Key: "A", Value: "standardA"},
@@ -102,19 +114,23 @@ func TestGetPolicies(t *testing.T) {
 		objectClass        ad.ObjectClass
 		userKrb5CCBaseName string
 		versionID          string
+		staticServerURL    string
+		domain             string
 
 		dontCreateOriginalKrb5CCName bool
 		turnKrb5CCRO                 bool
 
-		want    []entry.GPO
-		wantErr bool
+		want          []entry.GPO
+		wantServerURL string
+		wantErr       bool
 	}{
 		"Standard policy, user object": {
 			gpoListArgs:        "standard",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want:               []entry.GPO{standardGPO},
+			wantServerURL:      "ldap://myserver.example.com",
 		},
 		"Standard policy, computer object": {
 			gpoListArgs:        "standard",
@@ -128,10 +144,12 @@ func TestGetPolicies(t *testing.T) {
 						{Key: "D", Value: "standardD"},
 						{Key: "E", Value: "standardE"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"User only policy, user object": {
 			gpoListArgs:        "user-only",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -140,20 +158,24 @@ func TestGetPolicies(t *testing.T) {
 						{Key: "A", Value: "userOnlyA"},
 						{Key: "B", Value: "userOnlyB"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"User only policy, computer object, policy is empty": {
 			gpoListArgs:        "user-only",
 			objectName:         hostname,
 			objectClass:        ad.ComputerObject,
 			userKrb5CCBaseName: "",
 			want:               []entry.GPO{{ID: "user-only", Name: "user-only-name", Rules: make(map[string][]entry.Entry)}},
+			wantServerURL:      "ldap://myserver.example.com",
 		},
 		"Computer only policy, user object, policy is empty": {
 			gpoListArgs:        "machine-only",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want:               []entry.GPO{{ID: "machine-only", Name: "machine-only-name", Rules: make(map[string][]entry.Entry)}},
+			wantServerURL:      "ldap://myserver.example.com",
 		},
 		"Computer ignored CCBaseName": {
 			gpoListArgs:        "standard",
@@ -167,71 +189,83 @@ func TestGetPolicies(t *testing.T) {
 						{Key: "D", Value: "standardD"},
 						{Key: "E", Value: "standardE"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 
 		// Multi releases cases
 		"Enabled override": {
 			versionID:          "21.04",
 			gpoListArgs:        "multiple-releases-one-enabled",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{{ID: "multiple-releases-one-enabled", Name: "multiple-releases-one-enabled-name", Rules: map[string][]entry.Entry{
 				"dconf": {
 					{Key: "A", Value: "21.04Value"},
 				}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Disabled override": {
 			versionID:          "21.04",
 			gpoListArgs:        "multiple-releases-one-disabled",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{{ID: "multiple-releases-one-disabled", Name: "multiple-releases-one-disabled-name", Rules: map[string][]entry.Entry{
 				"dconf": {
 					{Key: "A", Value: "AllValue"},
 				}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Enabled override for matching release, other releases override ignored": {
 			versionID:          "21.04",
 			gpoListArgs:        "multiple-releases",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{{ID: "multiple-releases", Name: "multiple-releases-name", Rules: map[string][]entry.Entry{
 				"dconf": {
 					{Key: "A", Value: "21.04Value"},
 				}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Disable override for matching release, other releases override ignored": {
 			versionID:          "20.04",
 			gpoListArgs:        "multiple-releases",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{{ID: "multiple-releases", Name: "multiple-releases-name", Rules: map[string][]entry.Entry{
 				"dconf": {
 					{Key: "A", Value: "AllValue"},
 				}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"No override for this release, takes default value": {
 			versionID:          "not in pol file",
 			gpoListArgs:        "multiple-releases",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{{ID: "multiple-releases", Name: "multiple-releases-name", Rules: map[string][]entry.Entry{
 				"dconf": {
 					{Key: "A", Value: "AllValue"},
 				}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 
 		// No override option for this release
 
 		// Multi domain cases
 		"Multiple domains, same GPO": {
 			gpoListArgs:        "multiple-domains",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -243,10 +277,12 @@ func TestGetPolicies(t *testing.T) {
 					"other": {
 						{Key: "B", Value: "standardB"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Same key in different domains are kept separated": {
 			gpoListArgs:        "other-domain_one-value",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -258,11 +294,13 @@ func TestGetPolicies(t *testing.T) {
 					"dconf": {
 						{Key: "C", Value: "oneValueC"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 
 		"Two policies, with overrides": {
 			gpoListArgs:        "one-value_standard",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -277,10 +315,12 @@ func TestGetPolicies(t *testing.T) {
 						// this value will be overridden with the higher one
 						{Key: "C", Value: "standardC"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Two policies, with reversed overrides": {
 			gpoListArgs:        "standard_one-value",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -290,10 +330,12 @@ func TestGetPolicies(t *testing.T) {
 						// this value will be overridden with the higher one
 						{Key: "C", Value: "oneValueC"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Two policies, no overrides": {
 			gpoListArgs:        "one-value_user-only",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -306,10 +348,12 @@ func TestGetPolicies(t *testing.T) {
 						{Key: "A", Value: "userOnlyA"},
 						{Key: "B", Value: "userOnlyB"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Two policies, no overrides, reversed": {
 			gpoListArgs:        "user-only_one-value",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -322,19 +366,22 @@ func TestGetPolicies(t *testing.T) {
 					"dconf": {
 						{Key: "C", Value: "oneValueC"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Two policies, no overrides, one is not the same object type, machine ones are empty when parsing user": {
 			gpoListArgs:        "machine-only_standard",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{{ID: "machine-only", Name: "machine-only-name", Rules: make(map[string][]entry.Entry)},
 				standardGPO},
+			wantServerURL: "ldap://myserver.example.com",
 		},
 
 		"Disabled value overrides non disabled one": {
 			gpoListArgs:        "disabled-value_standard",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -343,10 +390,12 @@ func TestGetPolicies(t *testing.T) {
 						{Key: "C", Value: "", Disabled: true},
 					}}},
 				standardGPO,
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Disabled value is overridden": {
 			gpoListArgs:        "standard_disabled-value",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -355,11 +404,13 @@ func TestGetPolicies(t *testing.T) {
 					"dconf": {
 						{Key: "C", Value: "", Disabled: true},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 
 		"More policies, with multiple overrides": {
 			gpoListArgs:        "user-only_one-value_standard",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -373,17 +424,20 @@ func TestGetPolicies(t *testing.T) {
 						{Key: "C", Value: "oneValueC"},
 					}}},
 				standardGPO,
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
 		"Object domain is stripped": {
 			gpoListArgs:        "standard",
-			objectName:         "bob@warthogs.biz",
+			objectName:         "bob@example.com",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want:               []entry.GPO{standardGPO},
+			wantServerURL:      "ldap://myserver.example.com",
 		},
 		"Filter non Ubuntu keys": {
 			gpoListArgs:        "filtered",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			want: []entry.GPO{
@@ -392,7 +446,19 @@ func TestGetPolicies(t *testing.T) {
 						{Key: "A", Value: "standardA"},
 						{Key: "C", Value: "standardC"},
 					}}},
-			}},
+			},
+			wantServerURL: "ldap://myserver.example.com",
+		},
+
+		"No discovery for statistically configured domain controller": {
+			gpoListArgs:        "standard",
+			objectName:         "bob@EXAMPLE.COM",
+			objectClass:        ad.UserObject,
+			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
+			staticServerURL:    "ldap://myotherserver.example.com",
+			want:               []entry.GPO{standardGPO},
+			wantServerURL:      "ldap://myotherserver.example.com",
+		},
 
 		// Error cases
 		"Machine doesn’t match": {
@@ -404,14 +470,14 @@ func TestGetPolicies(t *testing.T) {
 		},
 		"Without previous call, needs userKrb5CCBaseName": {
 			gpoListArgs:        "standard",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "",
 			wantErr:            true,
 		},
 		"Unexisting CC original file for user": {
 			gpoListArgs:                  "standard",
-			objectName:                   "bob@WARTHOGS.BIZ",
+			objectName:                   "bob@EXAMPLE.COM",
 			objectClass:                  ad.UserObject,
 			userKrb5CCBaseName:           "kbr5cc_adsys_tests_bob",
 			dontCreateOriginalKrb5CCName: true,
@@ -424,23 +490,59 @@ func TestGetPolicies(t *testing.T) {
 			dontCreateOriginalKrb5CCName: true,
 			wantErr:                      true,
 		},
+		"No Active Directory server returned by sssd fails without static configuration": {
+			gpoListArgs:        "standard",
+			objectName:         "bob@EXAMPLE.COM",
+			objectClass:        ad.UserObject,
+			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
+			domain:             "emptyserver",
+			wantErr:            true,
+		},
+		"SSSD dbus (IsOnline) call failed": {
+			gpoListArgs:        "standard",
+			objectName:         "bob@EXAMPLE.COM",
+			objectClass:        ad.UserObject,
+			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
+			domain:             "unexistingdbusobjectdomain",
+			wantErr:            true,
+		},
+		// We can’t return an error from the dbus server objects without having a deadlock from godbus
+		// This affects testing only.
+		/*"SSSD ActiveServer call failed": {
+			gpoListArgs:        "standard",
+			objectName:         "bob@EXAMPLE.COM",
+			objectClass:        ad.UserObject,
+			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
+			domain:             "sssdactiveservercallfail",
+			wantErr:            true,
+		},
+		*/
+		"Error on user without @ in name": {
+			gpoListArgs:        "standard",
+			objectName:         "bob",
+			objectClass:        ad.UserObject,
+			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
+			want:               []entry.GPO{standardGPO},
+			wantServerURL:      "ldap://myserver.example.com",
+			wantErr:            true,
+		},
 		"Corrupted policy file": {
 			gpoListArgs:        "corrupted-policy",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			wantErr:            true,
 		},
 		"Policy can’t be downloaded": {
 			gpoListArgs:        "no-gpt-ini",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			wantErr:            true,
 		},
 		"Symlinks can’t be created": {
 			gpoListArgs:        "standard",
-			objectName:         "bob@WARTHOGS.BIZ",
+			objectName:         "bob@EXAMPLE.COM",
 			objectClass:        ad.UserObject,
 			userKrb5CCBaseName: "kbr5cc_adsys_tests_bob",
 			turnKrb5CCRO:       true,
@@ -467,8 +569,12 @@ func TestGetPolicies(t *testing.T) {
 				}
 			}
 
+			if tc.domain == "" {
+				tc.domain = "example.com"
+			}
+
 			cachedir, rundir := t.TempDir(), t.TempDir()
-			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "warthogs.biz",
+			adc, err := ad.New(context.Background(), tc.staticServerURL, tc.domain, bus,
 				ad.WithCacheDir(cachedir), ad.WithRunDir(rundir), ad.WithoutKerberos(),
 				ad.WithSSSCacheDir(sssCacheDir),
 				ad.WithGPOListCmd(mockGPOListCmd(t, tc.gpoListArgs)),
@@ -487,10 +593,16 @@ func TestGetPolicies(t *testing.T) {
 			entries, err := adc.GetPolicies(context.Background(), tc.objectName, tc.objectClass, krb5CCName)
 			if tc.wantErr {
 				require.NotNil(t, err, "GetPolicies should have errored out")
-			} else {
-				require.NoError(t, err, "GetPolicies should return no error")
+				return
 			}
+
+			require.NoError(t, err, "GetPolicies should return no error")
 			require.Equal(t, tc.want, entries, "GetPolicies returns expected gpo entries in correct order")
+			serverURL, isOffline := adc.GetStatus()
+			assert.False(t, isOffline, "We report that we are online")
+			if tc.staticServerURL == "" {
+				assert.Equal(t, tc.wantServerURL, serverURL, "Server URL has been fetched dynamically")
+			}
 		})
 	}
 }
@@ -498,6 +610,8 @@ func TestGetPolicies(t *testing.T) {
 func TestGetPoliciesOffline(t *testing.T) {
 	t.Parallel()
 
+	bus := ad.GetSystemBus(t)
+
 	gpos := []entry.GPO{
 		{ID: "user-only", Name: "user-only-name", Rules: map[string][]entry.Entry{
 			"dconf": {
@@ -517,16 +631,35 @@ func TestGetPoliciesOffline(t *testing.T) {
 
 	tests := map[string]struct {
 		getFromCache bool
+		domain       string
+		gpoListArg   string
 
 		want    []entry.GPO
 		wantErr bool
 	}{
 		"Offline, get from cache": {
 			getFromCache: true,
+			domain:       "offline",
+			gpoListArg:   "standard",
 			want:         gpos,
 		},
+		"Offline, ensure we fetch from cache and not fetch GPO list": {
+			getFromCache: true,
+			domain:       "offline",
+			gpoListArg:   "-Exit2-", // this should not be used
+			want:         gpos,
+		},
+
+		"Error on SSSD reports online, but we are actually offline when fetching gpo list, even with a cache": {
+			getFromCache: true,
+			domain:       "example.com",
+			gpoListArg:   "-Exit2-",
+			wantErr:      true,
+		},
 		"Error offline with no cache": {
 			getFromCache: false,
+			domain:       "offline",
+			gpoListArg:   "standard",
 			wantErr:      true,
 		},
 	}
@@ -537,12 +670,12 @@ func TestGetPoliciesOffline(t *testing.T) {
 			t.Parallel()
 
 			cachedir, rundir := t.TempDir(), t.TempDir()
-			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "warthogs.biz",
+			adc, err := ad.New(context.Background(), "", tc.domain, bus,
 				ad.WithCacheDir(cachedir), ad.WithRunDir(rundir), ad.WithoutKerberos(),
-				ad.WithGPOListCmd(mockGPOListCmd(t, "-Exit2-")))
+				ad.WithGPOListCmd(mockGPOListCmd(t, tc.gpoListArg)))
 			require.NoError(t, err, "Setup: cannot create ad object")
 
-			objectName := "useroffline@WARTHOGS.BIZ"
+			objectName := "useroffline@EXAMPLE.COM"
 
 			krb5CCName := setKrb5CC(t, objectName)
 
@@ -559,6 +692,9 @@ func TestGetPoliciesOffline(t *testing.T) {
 
 			require.NoError(t, err, "GetPolicies should return no error")
 			require.Equal(t, tc.want, entries, "GetPolicies returns expected gpo entries in correct order")
+			serverURL, isOffline := adc.GetStatus()
+			assert.True(t, isOffline, "We report that we are offline")
+			assert.Empty(t, serverURL, "Server URL has not been fetched in offline mode")
 		})
 	}
 }
@@ -566,6 +702,8 @@ func TestGetPoliciesOffline(t *testing.T) {
 func TestGetPoliciesWorkflows(t *testing.T) {
 	t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
 
+	bus := ad.GetSystemBus(t)
+
 	gpoListArgs := "standard"
 	objectClass := ad.UserObject
 
@@ -587,30 +725,30 @@ func TestGetPoliciesWorkflows(t *testing.T) {
 		wantErr bool
 	}{
 		"Second call is a refresh (without Krb5CCName specified)": {
-			objectName1:         "bob@WARTHOGS.BIZ",
-			objectName2:         "bob@WARTHOGS.BIZ",
+			objectName1:         "bob@EXAMPLE.COM",
+			objectName2:         "bob@EXAMPLE.COM",
 			userKrb5CCBaseName1: "bob",
 			userKrb5CCBaseName2: "EMPTY",
 			want:                []entry.GPO{standardGPO},
 		},
 		"Second call after service restarted": {
 			restart:             true,
-			objectName1:         "bob@WARTHOGS.BIZ",
-			objectName2:         "bob@WARTHOGS.BIZ",
+			objectName1:         "bob@EXAMPLE.COM",
+			objectName2:         "bob@EXAMPLE.COM",
 			userKrb5CCBaseName1: "bob",
 			userKrb5CCBaseName2: "", // We did’t RENEW the ticket
 			want:                []entry.GPO{standardGPO},
 		},
 		"Second call with different user": {
-			objectName1:         "bob@WARTHOGS.BIZ",
-			objectName2:         "sponge@WARTHOGS.BIZ",
+			objectName1:         "bob@EXAMPLE.COM",
+			objectName2:         "sponge@EXAMPLE.COM",
 			userKrb5CCBaseName1: "bob",
 			userKrb5CCBaseName2: "sponge",
 			want:                []entry.GPO{standardGPO},
 		},
 		"Second call after a relogin": {
-			objectName1:         "bob@WARTHOGS.BIZ",
-			objectName2:         "bob@WARTHOGS.BIZ",
+			objectName1:         "bob@EXAMPLE.COM",
+			objectName2:         "bob@EXAMPLE.COM",
 			userKrb5CCBaseName1: "bob",
 			userKrb5CCBaseName2: "bobNew",
 			want:                []entry.GPO{standardGPO},
@@ -626,7 +764,7 @@ func TestGetPoliciesWorkflows(t *testing.T) {
 
 			cachedir, rundir := t.TempDir(), t.TempDir()
 
-			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "warthogs.biz",
+			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "example.com", bus,
 				ad.WithCacheDir(cachedir), ad.WithRunDir(rundir), ad.WithoutKerberos(),
 				ad.WithSSSCacheDir("testdata/sss/db"),
 				ad.WithGPOListCmd(mockGPOListCmd(t, gpoListArgs)))
@@ -648,7 +786,7 @@ func TestGetPoliciesWorkflows(t *testing.T) {
 
 			// Restart: recreate ad object
 			if tc.restart {
-				adc, err = ad.New(context.Background(), "ldap://UNUSED:1636/", "warthogs.biz",
+				adc, err = ad.New(context.Background(), "ldap://UNUSED:1636/", "example.com", bus,
 					ad.WithCacheDir(cachedir), ad.WithRunDir(rundir), ad.WithoutKerberos(),
 					ad.WithSSSCacheDir("testdata/sss/db"),
 					ad.WithGPOListCmd(mockGPOListCmd(t, gpoListArgs)))
@@ -666,6 +804,8 @@ func TestGetPoliciesWorkflows(t *testing.T) {
 func TestGetPoliciesConcurrently(t *testing.T) {
 	t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
 
+	bus := ad.GetSystemBus(t)
+
 	objectClass := ad.UserObject
 
 	standardGPO := entry.GPO{ID: "standard", Name: "standard-name", Rules: map[string][]entry.Entry{
@@ -686,8 +826,8 @@ func TestGetPoliciesConcurrently(t *testing.T) {
 		wantErr bool
 	}{
 		"Same user, same GPO": {
-			objectName1: "bob@WARTHOGS.BIZ",
-			objectName2: "bob@WARTHOGS.BIZ",
+			objectName1: "bob@EXAMPLE.COM",
+			objectName2: "bob@EXAMPLE.COM",
 			gpo1:        "standard",
 			gpo2:        "standard",
 			want1:       []entry.GPO{standardGPO},
@@ -696,8 +836,8 @@ func TestGetPoliciesConcurrently(t *testing.T) {
 		// We can’t run this test currently as the mock will always return the same value for bob (both gpos):
 		// both calls are identical.
 		/*"Same user, different GPOs": {
-		objectName1: "bob@WARTHOGS.BIZ",
-		objectName2: "bob@WARTHOGS.BIZ",
+		objectName1: "bob@EXAMPLE.COM",
+		objectName2: "bob@EXAMPLE.COM",
 		gpo1:        "standard",
 		gpo2:        "one-value",
 		want1: []entry.GPO{standardGPO},
@@ -707,16 +847,16 @@ func TestGetPoliciesConcurrently(t *testing.T) {
 				}}},
 			}},*/
 		"Different users, same GPO": {
-			objectName1: "bob@WARTHOGS.BIZ",
-			objectName2: "sponge@WARTHOGS.BIZ",
+			objectName1: "bob@EXAMPLE.COM",
+			objectName2: "sponge@EXAMPLE.COM",
 			gpo1:        "standard",
 			gpo2:        "standard",
 			want1:       []entry.GPO{standardGPO},
 			want2:       []entry.GPO{standardGPO},
 		},
 		"Different users, different GPO": {
-			objectName1: "bob@WARTHOGS.BIZ",
-			objectName2: "sponge@WARTHOGS.BIZ",
+			objectName1: "bob@EXAMPLE.COM",
+			objectName2: "sponge@EXAMPLE.COM",
 			gpo1:        "standard",
 			gpo2:        "one-value",
 			want1:       []entry.GPO{standardGPO},
@@ -750,7 +890,7 @@ func TestGetPoliciesConcurrently(t *testing.T) {
 			if mockObjectName1 == mockObjectName2 {
 				gpoListMeta = fmt.Sprintf("DEPENDS:%s@%s", mockObjectName1, tc.gpo1)
 			}
-			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "warthogs.biz",
+			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "example.com", bus,
 				ad.WithCacheDir(cachedir), ad.WithRunDir(rundir), ad.WithoutKerberos(),
 				ad.WithSSSCacheDir("testdata/sss/db"),
 				ad.WithGPOListCmd(mockGPOListCmd(t, gpoListMeta)))
@@ -778,6 +918,8 @@ func TestGetPoliciesConcurrently(t *testing.T) {
 func TestListUsersFromCache(t *testing.T) {
 	t.Parallel()
 
+	bus := ad.GetSystemBus(t)
+
 	tests := map[string]struct {
 		ccCachesToCreate []string
 		noCCacheDir      bool
@@ -786,20 +928,20 @@ func TestListUsersFromCache(t *testing.T) {
 		wantErr bool
 	}{
 		"One user": {
-			ccCachesToCreate: []string{"bob@WARTHOGS.BIZ"},
-			want:             []string{"bob@WARTHOGS.BIZ"},
+			ccCachesToCreate: []string{"bob@EXAMPLE.COM"},
+			want:             []string{"bob@EXAMPLE.COM"},
 		},
 		"Two users": {
-			ccCachesToCreate: []string{"bob@WARTHOGS.BIZ", "sponge@OTHERDOMAIN.BIZ"},
-			want:             []string{"bob@WARTHOGS.BIZ", "sponge@OTHERDOMAIN.BIZ"},
+			ccCachesToCreate: []string{"bob@EXAMPLE.COM", "sponge@OTHERDOMAIN.BIZ"},
+			want:             []string{"bob@EXAMPLE.COM", "sponge@OTHERDOMAIN.BIZ"},
 		},
 		"None": {
 			ccCachesToCreate: []string{},
 			want:             nil,
 		},
 		"Machines are ignored": {
-			ccCachesToCreate: []string{"bob@WARTHOGS.BIZ", "sponge@OTHERDOMAIN.BIZ", "myMachine"},
-			want:             []string{"bob@WARTHOGS.BIZ", "sponge@OTHERDOMAIN.BIZ"},
+			ccCachesToCreate: []string{"bob@EXAMPLE.COM", "sponge@OTHERDOMAIN.BIZ", "myMachine"},
+			want:             []string{"bob@EXAMPLE.COM", "sponge@OTHERDOMAIN.BIZ"},
 		},
 		"Machine Only": {
 			ccCachesToCreate: []string{"myMachine"},
@@ -830,7 +972,7 @@ func TestListUsersFromCache(t *testing.T) {
 				}
 			}
 
-			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "warthogs.biz",
+			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "example.com", bus,
 				ad.WithCacheDir(cachedir), ad.WithRunDir(rundir))
 			require.NoError(t, err, "Setup: New should return no error")
 
@@ -838,7 +980,7 @@ func TestListUsersFromCache(t *testing.T) {
 				require.NoError(t, os.RemoveAll(krb5CacheDir), "Setup: can’t remove krb5 cache directory")
 			}
 
-			got, err := adc.ListUsersFromCache(context.Background())
+			got, err := adc.ListActiveUsers(context.Background())
 			if tc.wantErr {
 				require.Error(t, err, "ListUsersFromCache should return an error and didn't")
 				return
@@ -852,6 +994,63 @@ func TestListUsersFromCache(t *testing.T) {
 	}
 }
 
+func TestNormalizeTargetName(t *testing.T) {
+	t.Parallel()
+
+	hostname, err := os.Hostname()
+	require.NoError(t, err, "Setup: failed to get hostname")
+
+	bus := ad.GetSystemBus(t)
+
+	tests := map[string]struct {
+		target              string
+		objectClass         ad.ObjectClass
+		defaultDomainSuffix string
+
+		want    string
+		wantErr bool
+	}{
+		"One valid user":                          {target: "user@example.com", want: "user@example.com"},
+		`One valid user with domain\user`:         {target: `example.com\user`, want: "user@example.com"},
+		"One user without explicit domain suffix": {target: "user", defaultDomainSuffix: "example.com", want: "user@example.com"},
+
+		// User match computer names
+		"User name matching computer, setting as user": {target: hostname, objectClass: ad.UserObject, defaultDomainSuffix: "example.com",
+			want: hostname + "@example.com"},
+		"User name fqdn matching computer":  {target: hostname + "@example.com", want: hostname + "@example.com"},
+		"Computer name without objectClass": {target: hostname, want: hostname},
+
+		// Computer cases
+		"Computer is left as such":        {target: "computername", objectClass: ad.ComputerObject, want: "computername"},
+		"Computer with @ is left as such": {target: "computername@example.com", objectClass: ad.ComputerObject, want: "computername@example.com"},
+
+		// Error cases
+		`Error on multiple \ in name`:                        {target: `example.com\user\something`, wantErr: true},
+		`Error on no default domain suffix and no fqdn user`: {target: `user`, wantErr: true},
+	}
+
+	for name, tc := range tests {
+		tc := tc
+		t.Run(name, func(t *testing.T) {
+			t.Parallel()
+
+			adc, err := ad.New(context.Background(), "ldap://UNUSED:1636/", "example.com", bus,
+				ad.WithCacheDir(t.TempDir()), ad.WithRunDir(t.TempDir()),
+				ad.WithDefaultDomainSuffix(tc.defaultDomainSuffix))
+			require.NoError(t, err, "Setup: New should return no error")
+
+			got, err := adc.NormalizeTargetName(context.Background(), tc.target, tc.objectClass)
+			if tc.wantErr {
+				require.Error(t, err, "NormalizeTargetName should return an error and didn't")
+				return
+			}
+			require.NoError(t, err, "NormalizeTargetName should return no error")
+
+			assert.Equal(t, tc.want, got, "NormalizeTargetName should return the expected user")
+		})
+	}
+}
+
 func TestMockGPOList(t *testing.T) {
 	if os.Getenv("GO_WANT_HELPER_PROCESS") != "1" {
 		return
@@ -884,22 +1083,19 @@ func TestMockGPOList(t *testing.T) {
 		os.Exit(2)
 	}
 
-	// domain shouldn’t be used on object name, as we will return nothing
-	if strings.Contains(args[len(args)-1], "@") {
-		return
-	}
+	// as in gpolist, we split on the @ if any
+	objectName := args[len(args)-1]
+	objectName = strings.Split(objectName, "@")[0]
 
 	var gpos []string
 
 	// Parameterized on user gpos
 	if strings.HasPrefix(args[0], "DEPENDS:") {
-		// user is the last argument of the list command
-		user := args[len(args)-1]
 		v := strings.TrimPrefix(args[0], "DEPENDS:")
 		gpoItems := strings.Split(v, ":")
 		for _, gpoItem := range gpoItems {
 			i := strings.SplitN(gpoItem, "@", 2)
-			if i[0] == user {
+			if i[0] == objectName {
 				gpos = append(gpos, i[1])
 			}
 		}
@@ -908,7 +1104,7 @@ func TestMockGPOList(t *testing.T) {
 	}
 
 	for _, gpo := range gpos {
-		fmt.Fprintf(os.Stdout, "%s-name\tsmb://localhost:%d/SYSVOL/warthogs.biz/Policies/%s\n", gpo, ad.SmbPort, gpo)
+		fmt.Fprintf(os.Stdout, "%s-name\tsmb://localhost:%d/SYSVOL/example.com/Policies/%s\n", gpo, ad.SmbPort, gpo)
 	}
 }
 
