@@ -5,18 +5,15 @@ import (
 	"crypto/md5"
 	"flag"
 	"fmt"
-	"io"
 	"log"
-	"net"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"strings"
 	"sync"
 	"testing"
-	"text/template"
-	"time"
 
+	"github.com/godbus/dbus/v5"
+	"github.com/godbus/dbus/v5/introspect"
 	"github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -31,6 +28,8 @@ var Update bool
 func TestFetchGPO(t *testing.T) {
 	t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
 
+	bus := GetSystemBus(t)
+
 	tests := map[string]struct {
 		gpos                   []string
 		concurrentGposDownload []string
@@ -192,8 +191,8 @@ func TestFetchGPO(t *testing.T) {
 			t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
 			dest, rundir := t.TempDir(), t.TempDir()
 
-			adc, err := New(context.Background(), "ldap://UNUSED:1636/", "localdomain",
-				WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), withSSSCacheDir("testdata/sss/db"))
+			adc, err := New(context.Background(), "ldap://UNUSED:1636/", "localdomain", bus,
+				WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), WithSSSCacheDir("testdata/sss/db"))
 
 			require.NoError(t, err, "Setup: cannot create ad object")
 
@@ -273,6 +272,8 @@ func TestFetchGPO(t *testing.T) {
 func TestFetchGPOWithUnreadableFile(t *testing.T) {
 	t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
 
+	bus := GetSystemBus(t)
+
 	// Prepare GPO with unreadable file.
 	// Defer will work after all tests are done because we don’t run it in parallel
 	gpos := map[string]string{
@@ -303,8 +304,8 @@ func TestFetchGPOWithUnreadableFile(t *testing.T) {
 
 			dest, rundir := t.TempDir(), t.TempDir()
 
-			adc, err := New(context.Background(), "ldap://UNUSED:1636/", "localdomain",
-				WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), withSSSCacheDir("testdata/sss/db"))
+			adc, err := New(context.Background(), "ldap://UNUSED:1636/", "localdomain", bus,
+				WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), WithSSSCacheDir("testdata/sss/db"))
 			require.NoError(t, err, "Setup: cannot create ad object")
 
 			if tc.withExistingGPO {
@@ -335,6 +336,9 @@ func TestFetchGPOWithUnreadableFile(t *testing.T) {
 
 func TestFetchGPOTweakGPOCacheDir(t *testing.T) {
 	t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
+
+	bus := GetSystemBus(t)
+
 	tests := map[string]struct {
 		removeGPOCacheDir bool
 		roGPOCacheDir     bool
@@ -349,8 +353,8 @@ func TestFetchGPOTweakGPOCacheDir(t *testing.T) {
 			t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
 
 			dest, rundir := t.TempDir(), t.TempDir()
-			adc, err := New(context.Background(), "ldap://UNUSED:1636/", "localdomain",
-				WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), withSSSCacheDir("testdata/sss/db"))
+			adc, err := New(context.Background(), "ldap://UNUSED:1636/", "localdomain", bus,
+				WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), WithSSSCacheDir("testdata/sss/db"))
 			require.NoError(t, err, "Setup: cannot create ad object")
 
 			if tc.removeGPOCacheDir {
@@ -371,11 +375,13 @@ func TestFetchGPOTweakGPOCacheDir(t *testing.T) {
 func TestFetchOneGPOWhileParsingItConcurrently(t *testing.T) {
 	t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
 
-	const policyPath = "SYSVOL/warthogs.biz/Policies"
+	bus := GetSystemBus(t)
+
+	const policyPath = "SYSVOL/example.com/Policies"
 	dest, rundir := t.TempDir(), t.TempDir()
 
-	adc, err := New(context.Background(), "ldap://UNUSED:1636/", "warthogs.biz",
-		WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), withSSSCacheDir("testdata/sss/db"))
+	adc, err := New(context.Background(), "ldap://UNUSED:1636/", "example.com", bus,
+		WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), WithSSSCacheDir("testdata/sss/db"))
 	require.NoError(t, err, "Setup: cannot create ad object")
 
 	// ensure the GPO is already downloaded with an older version to force redownload
@@ -418,11 +424,13 @@ func TestFetchOneGPOWhileParsingItConcurrently(t *testing.T) {
 func TestParseGPOConcurrent(t *testing.T) {
 	t.Parallel() // libsmbclient overrides SIGCHILD, but we have one global lock
 
-	const policyPath = "SYSVOL/warthogs.biz/Policies"
+	bus := GetSystemBus(t)
+
+	const policyPath = "SYSVOL/example.com/Policies"
 	dest, rundir := t.TempDir(), t.TempDir()
 
-	adc, err := New(context.Background(), "ldap://UNUSED:1636/", "warthogs.biz",
-		WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), withSSSCacheDir("testdata/sss/db"))
+	adc, err := New(context.Background(), "ldap://UNUSED:1636/", "example.com", bus,
+		WithCacheDir(dest), WithRunDir(rundir), withoutKerberos(), WithSSSCacheDir("testdata/sss/db"))
 	require.NoError(t, err, "Setup: cannot create ad object")
 
 	// Fetch the GPO to set it up
@@ -447,151 +455,107 @@ func TestParseGPOConcurrent(t *testing.T) {
 	wg.Wait()
 }
 
+const SmbPort = 1445
+
 var brokenSmbDirShare string
 
-const (
-	SmbPort         = 1445
-	smbConfTemplate = `[global]
-workgroup = TESTGROUP
-interfaces = lo 127.0.0.0/8
-smb ports = {{.SmbPort}}
-log level = 2
-map to guest = Bad User
-passdb backend = smbpasswd
-smb passwd file = {{.Tempdir}}/smbpasswd
-lock directory = {{.Tempdir}}/intern
-state directory = {{.Tempdir}}/intern
-cache directory = {{.Tempdir}}/intern
-pid directory = {{.Tempdir}}/intern
-private dir = {{.Tempdir}}/intern
-ncalrpc dir = {{.Tempdir}}/intern
-
-[SYSVOL]
-path = {{.Cwd}}/testdata/AD/SYSVOL
-guest ok = yes
-
-[broken]
-path = {{.BrokenRoot}}
-guest ok = yes
-`
-)
-
-func mkSmbDir() (string, func()) {
-	dir, err := os.MkdirTemp("", "adsys_smbd_")
-	if err != nil {
-		log.Fatalf("Setup: failed to create temporary smb directory: %v", err)
-	}
-
-	brokenSmbDirShare, err = os.MkdirTemp("", "adsys_smbd_broken_share_")
-	if err != nil {
-		log.Fatalf("Setup: failed to create temporary broken smb share directory: %v", err)
-	}
-	if err = os.MkdirAll(filepath.Join(brokenSmbDirShare, policyPath), 0700); err != nil {
-		log.Fatalf("Setup: failed to created temporary broken smb share AD structure: %v", err)
-	}
-
-	type smbConfVars struct {
-		Tempdir    string
-		Cwd        string
-		SmbPort    int
-		BrokenRoot string
-	}
-	t, err := template.New("smb-conf").Parse(smbConfTemplate)
-	if err != nil {
-		log.Fatalf("Setup: can’t open template for smbd configuration: %v", err)
-	}
-
-	f, err := os.Create(filepath.Join(dir, "smbd.conf"))
-	if err != nil {
-		log.Fatalf("Setup: can’t create smbd configuration: %v", err)
-	}
-	defer f.Close()
-
-	cwd, err := os.Getwd()
-	if err != nil {
-		log.Fatalf("Setup: can’t determine current work directory: %v", err)
-	}
-	if err := t.Execute(f, smbConfVars{Tempdir: dir, Cwd: cwd, SmbPort: SmbPort, BrokenRoot: brokenSmbDirShare}); err != nil {
-		log.Fatalf("Setup: failed to create smb.conf: %v", err)
-	}
-
-	return dir, func() {
-		os.RemoveAll(dir)
-	}
-}
-
 func TestMain(m *testing.M) {
 	flag.BoolVar(&Update, "update", false, "update golden files")
 	flag.Parse()
 
-	// Don’t setup samba for mock helpers
+	// Don’t setup samba or sssd for mock helpers
 	if !strings.Contains(strings.Join(os.Args, " "), "TestMock") {
 		debug := flag.Bool("verbose", false, "Print debug log level information within the test")
 		flag.Parse()
 		if *debug {
 			logrus.StandardLogger().SetLevel(logrus.DebugLevel)
 		}
-		defer setupSmb()()
+
+		// Samba
+		var err error
+		brokenSmbDirShare, err = os.MkdirTemp("", "adsys_smbd_broken_share_")
+		if err != nil {
+			log.Fatalf("Setup: failed to create temporary broken smb share directory: %v", err)
+		}
+		if err = os.MkdirAll(filepath.Join(brokenSmbDirShare, policyPath), 0700); err != nil {
+			log.Fatalf("Setup: failed to created temporary broken smb share AD structure: %v", err)
+		}
+		defer func() {
+			if err := os.RemoveAll(brokenSmbDirShare); err != nil {
+				log.Fatalf("Teardown: failed to remove broken smb directory: %v", err)
+			}
+		}()
+		defer testutils.SetupSmb(SmbPort, "testdata/AD/SYSVOL", brokenSmbDirShare)()
+
+		// SSSD domains
+		defer testutils.StartLocalSystemBus()()
+
+		conn, err := dbus.SystemBusPrivate()
+		if err != nil {
+			log.Fatalf("Setup: can’t get a private system bus: %v", err)
+		}
+		defer func() {
+			if err = conn.Close(); err != nil {
+				log.Fatalf("Teardown: can’t close system dbus connection: %v", err)
+			}
+		}()
+		if err = conn.Auth(nil); err != nil {
+			log.Fatalf("Setup: can’t auth on private system bus: %v", err)
+		}
+		if err = conn.Hello(); err != nil {
+			log.Fatalf("Setup: can’t send hello message on private system bus: %v", err)
+		}
+
+		sssdOnlineExample := sssd{
+			domain: "example.com",
+			online: true,
+		}
+		sssdOnlineLocalDomain := sssd{
+			domain: "locadomain",
+			online: true,
+		}
+		sssdEmptyServerDomain := sssd{
+			domain: "",
+			online: true,
+		}
+		sssdOfflineExample := sssd{
+			domain: "example.com",
+			online: false,
+		}
+		const intro = `
+	<node>
+		<interface name="org.freedesktop.sssd.infopipe.Domains.Domain">
+			<method name="ActiveServer">
+				<arg direction="in" type="s"/>
+				<arg direction="out" type="s"/>
+			</method>
+			<method name="IsOnline">
+				<arg direction="out" type="b"/>
+			</method>
+		</interface>` + introspect.IntrospectDataString + `</node>`
+		conn.Export(sssdOnlineExample, "/org/freedesktop/sssd/infopipe/Domains/example_2ecom", "org.freedesktop.sssd.infopipe.Domains.Domain")
+		conn.Export(introspect.Introspectable(intro), "/org/freedesktop/sssd/infopipe/Domains/example_2ecom",
+			"org.freedesktop.DBus.Introspectable")
+		conn.Export(sssdOnlineLocalDomain, "/org/freedesktop/sssd/infopipe/Domains/localdomain", "org.freedesktop.sssd.infopipe.Domains.Domain")
+		conn.Export(introspect.Introspectable(intro), "/org/freedesktop/sssd/infopipe/Domains/localdomain",
+			"org.freedesktop.DBus.Introspectable")
+		conn.Export(sssdEmptyServerDomain, "/org/freedesktop/sssd/infopipe/Domains/emptyserver", "org.freedesktop.sssd.infopipe.Domains.Domain")
+		conn.Export(introspect.Introspectable(intro), "/org/freedesktop/sssd/infopipe/Domains/emptyserver",
+			"org.freedesktop.DBus.Introspectable")
+		conn.Export(sssdOfflineExample, "/org/freedesktop/sssd/infopipe/Domains/offline", "org.freedesktop.sssd.infopipe.Domains.Domain")
+		conn.Export(introspect.Introspectable(intro), "/org/freedesktop/sssd/infopipe/Domains/offline",
+			"org.freedesktop.DBus.Introspectable")
+
+		reply, err := conn.RequestName("org.freedesktop.sssd.infopipe", dbus.NameFlagDoNotQueue)
+		if err != nil {
+			log.Fatalf("Setup: Failed to acquire sssd name on local system bus: %v", err)
+		}
+		if reply != dbus.RequestNameReplyPrimaryOwner {
+			log.Fatalf("Setup: Failed to acquire sssd name on local system bus: name is already taken")
+		}
 	}
 	m.Run()
-	testutils.MergePythonCoverage()
-}
-
-func setupSmb() func() {
-	dir, cleanup := mkSmbDir()
-
-	cmd := exec.Command("smbd", "-FS", "-s", filepath.Join(dir, "smbd.conf"))
-	stderr, err := cmd.StderrPipe()
-	if err != nil {
-		log.Fatalf("Setup: can’t get smb output: %v", err)
-	}
-
-	if err := cmd.Start(); err != nil {
-		cleanup()
-		log.Fatalf("Setup: can’t start smb: %v", err)
-	}
-
-	waitForPortReady(SmbPort)
-	return func() {
-		if err := cmd.Process.Kill(); err != nil {
-			log.Fatalf("Setup: failed to kill smbd process: %v", err)
-		}
-
-		d, err := io.ReadAll(stderr)
-		if err != nil {
-			fmt.Fprintf(os.Stderr, "Setup: Can't show stderr from smbd command: %v", err)
-		}
-		if string(d) != "" {
-			fmt.Fprintf(os.Stderr, "Setup: samba output: %s\n", d)
-		}
-
-		if _, err = cmd.Process.Wait(); err != nil {
-			log.Fatalf("Setup: failed to wait for smbd: %v", err)
-		}
-
-		cleanup()
-	}
-}
-
-// waitForPortReady to be opened.
-func waitForPortReady(port int) {
-	timeout := time.NewTimer(5 * time.Second)
-	for {
-		select {
-		case <-timeout.C:
-			log.Fatalf("Setup: smbd hasn’t started successfully")
-		default:
-		}
-
-		conn, err := net.Dial("tcp", fmt.Sprintf("localhost:%d", port))
-		if err != nil {
-			time.Sleep(10 * time.Millisecond)
-			continue
-		}
-		conn.Close()
-		time.Sleep(10 * time.Millisecond)
-		return
-	}
+	testutils.MergeCoverages()
 }
 
 // md5Tree build a recursive file list of dir and with their md5sum
@@ -623,3 +587,32 @@ func md5Tree(t *testing.T, dir string) map[string]string {
 
 	return r
 }
+
+type sssd struct {
+	domain string
+	online bool
+}
+
+func (s sssd) ActiveServer(_ string) (string, *dbus.Error) {
+	if s.domain == "" {
+		return "", nil
+	}
+	return "myserver." + s.domain, nil
+}
+
+func (s sssd) IsOnline() (bool, *dbus.Error) {
+	return s.online, nil
+}
+
+// GetSystemBus helper functionality to get one system bus which automatically close on test shutdown
+func GetSystemBus(t *testing.T) *dbus.Conn {
+	// Don’t call dbus.SystemBus which caches globally system dbus (issues in tests)
+	bus, err := dbus.SystemBusPrivate()
+	require.NoError(t, err, "can’t get private system bus")
+	t.Cleanup(func() { _ = bus.Close() })
+	err = bus.Auth(nil)
+	require.NoError(t, err, "can’t auth on private system bus")
+	err = bus.Hello()
+	require.NoError(t, err, "can’t ping private system bus")
+	return bus
+}
