@@ -1,20 +1,13 @@
 package ad
 
-func withSSSCacheDir(cacheDir string) func(o *options) error {
-	return func(o *options) error {
-		o.sssCacheDir = cacheDir
-		return nil
-	}
-}
-
-func withoutKerberos() func(o *options) error {
+func withoutKerberos() Option {
 	return func(o *options) error {
 		o.withoutKerberos = true
 		return nil
 	}
 }
 
-func withGPOListCmd(cmd []string) func(o *options) error {
+func withGPOListCmd(cmd []string) Option {
 	return func(o *options) error {
 		o.gpoListCmd = cmd
 		return nil
@@ -22,7 +15,7 @@ func withGPOListCmd(cmd []string) func(o *options) error {
 }
 
 // WithVersionID specifies a personalized release id
-func WithVersionID(versionID string) func(o *options) error {
+func WithVersionID(versionID string) Option {
 	return func(o *options) error {
 		o.versionID = versionID
 		return nil
