@@ -30,13 +30,8 @@ func main() {
 			log.Fatal(err)
 		}
 		args := []string{"--shared", "-Wl,-soname,libpam_adsys.so"}
-		for _, flagType := range []string{"CPPFLAGS", "CFLAGS", "LDFLAGS"} {
-			for _, f := range strings.Split(os.Getenv(flagType), " ") {
-				if strings.TrimSpace(f) == "" {
-					continue
-				}
-				args = append(args, f)
-			}
+		for _, flagType := range []string{"CPPFLAGS", "CFLAGS"} {
+			args = appendFlagsToArgs(flagType, args)
 		}
 
 		_, curF, _, ok := runtime.Caller(0)
@@ -45,7 +40,8 @@ func main() {
 		}
 		dir := filepath.Dir(curF)
 
-		args = append(args, "-lpam", "-o", filepath.Join(destDir, "pam_adsys.so"), filepath.Join(dir, "pam_adsys.c"))
+		args = append(args, "-o", filepath.Join(destDir, "pam_adsys.so"), filepath.Join(dir, "pam_adsys.c"), "-lpam")
+		args = appendFlagsToArgs("LDFLAGS", args)
 		cmd := exec.Command("gcc", args...)
 		cmd.Stdout = os.Stdout
 		cmd.Stderr = os.Stderr
@@ -61,3 +57,13 @@ func main() {
 		}
 	}
 }
+
+func appendFlagsToArgs(flagType string, args []string) []string {
+	for _, f := range strings.Split(os.Getenv(flagType), " ") {
+		if strings.TrimSpace(f) == "" {
+			continue
+		}
+		args = append(args, f)
+	}
+	return args
+}
